@@ -31,17 +31,31 @@
 #include "ews-errors.h"
 #include "e-ews-folder.h"
 #include "e-ews-item.h"
+#include "camel-ews-settings.h"
+
+/* Standard GObject macros */
+#define E_TYPE_EWS_CONNECTION \
+	(e_ews_connection_get_type ())
+#define E_EWS_CONNECTION(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), E_TYPE_EWS_CONNECTION, EEwsConnection))
+#define E_EWS_CONNECTION_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), E_TYPE_EWS_CONNECTION, EEwsConnectionClass))
+#define E_IS_EWS_CONNECTION(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), E_TYPE_EWS_CONNECTION))
+#define E_IS_EWS_CONNECTION_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), E_TYPE_EWS_CONNECTION))
+#define E_EWS_CONNECTION_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), E_TYPE_EWS_CONNECTION, EEwsConnectionClass))
 
 G_BEGIN_DECLS
 
-#define E_TYPE_EWS_CONNECTION            (e_ews_connection_get_type ())
-#define E_EWS_CONNECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), E_TYPE_EWS_CONNECTION, EEwsConnection))
-#define E_EWS_CONNECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), E_TYPE_EWS_CONNECTION, EEwsConnectionClass))
-#define E_IS_EWS_CONNECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), E_TYPE_EWS_CONNECTION))
-#define E_IS_EWS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), E_TYPE_EWS_CONNECTION))
-
-typedef struct _EEwsConnection        EEwsConnection;
-typedef struct _EEwsConnectionClass   EEwsConnectionClass;
+typedef struct _EEwsConnection EEwsConnection;
+typedef struct _EEwsConnectionClass EEwsConnectionClass;
 typedef struct _EEwsConnectionPrivate EEwsConnectionPrivate;
 
 struct _EEwsConnection {
@@ -51,8 +65,6 @@ struct _EEwsConnection {
 
 struct _EEwsConnectionClass {
 	GObjectClass parent_class;
-
-	void (*authenticate) (EEwsConnection *cnc);
 };
 
 enum {
@@ -61,9 +73,12 @@ enum {
 	EWS_PRIORITY_HIGH
 };
 
-typedef void (*EEwsRequestCreationCallback) (ESoapMessage *msg,
-					     gpointer user_data);
-typedef void (*EwsProgressFn) (gpointer object, gint percent);
+typedef void	(*EEwsRequestCreationCallback)	(ESoapMessage *msg,
+						 gpointer user_data);
+typedef void	(*EwsProgressFn)		(gpointer object,
+						 gint percent);
+typedef void	(*EEwsResponseCallback)		(ESoapResponse *response,
+						 GSimpleAsyncResult *simple);
 
 typedef enum {
 	EWS_SEARCH_AD,
@@ -115,60 +130,143 @@ typedef struct {
 	gchar *filename;
 } EwsOALDetails;
 
+typedef struct {
+	gchar *sid;
+	gchar *primary_smtp;
+	gchar *display_name;
+	gchar *distinguished_user;
+	gchar *external_user;
+} EwsUserId;
+
+typedef enum {
+	EwsPermissionLevel_Unknown = 0,
+	EwsPermissionLevel_None,
+	EwsPermissionLevel_Reviewer,
+	EwsPermissionLevel_Author,
+	EwsPermissionLevel_Editor,
+	EwsPermissionLevel_Custom
+} EwsPermissionLevel;
+
+typedef struct {
+	EwsUserId *user_id;
+	EwsPermissionLevel calendar, tasks, inbox, contacts, notes, journal;
+	gboolean meetingcopies;
+	gboolean view_priv_items;
+} EwsDelegateInfo;
+
+typedef enum {
+	EwsDelegateDeliver_DelegatesOnly,
+	EwsDelegateDeliver_DelegatesAndMe,
+	EwsDelegateDeliver_DelegatesAndSendInformationToMe
+} EwsDelegateDeliver;
+
+typedef enum {
+	NORMAL_FIELD_URI,
+	INDEXED_FIELD_URI,
+	EXTENDED_FIELD_URI
+} EwsFieldURIType;
+
+typedef struct {
+	gchar *distinguished_prop_set_id;
+	gchar *prop_set_id;
+	gchar *prop_tag;
+	gchar *prop_name;
+	gchar *prop_id;
+	gchar *prop_type;
+} EwsExtendedFieldURI;
+
+typedef struct {
+	gchar *field_uri;
+	gchar *field_index;
+} EwsIndexedFieldURI;
+
+typedef struct {
+	gchar *field_uri;
+	GSList *extended_furis;
+	GSList *indexed_furis;
+} EwsAdditionalProps;
+
+typedef struct {
+	gchar *order;
+	gint uri_type;
+	gpointer field_uri;
+} EwsSortOrder;
+
+void		ews_oal_free			(EwsOAL *oal);
+void		ews_oal_details_free		(EwsOALDetails *details);
+
 GType		e_ews_connection_get_type	(void);
 EEwsConnection *e_ews_connection_new		(const gchar *uri,
-						 const gchar *username,
-						 const gchar *password,
-						 GCallback authenticate_cb,
-						 gpointer authenticate_ctx,
-						 GError **error);
+						 CamelEwsSettings *settings);
+const gchar *	e_ews_connection_get_uri	(EEwsConnection *cnc);
+const gchar *	e_ews_connection_get_password	(EEwsConnection *cnc);
+gchar *		e_ews_connection_dup_password	(EEwsConnection *cnc);
+void		e_ews_connection_set_password	(EEwsConnection *cnc,
+						 const gchar *password);
+CamelEwsSettings *
+		e_ews_connection_ref_settings	(EEwsConnection *cnc);
+SoupSession *	e_ews_connection_ref_soup_session
+						(EEwsConnection *cnc);
 EEwsConnection *e_ews_connection_find		(const gchar *uri,
 						 const gchar *username);
-void		e_ews_connection_authenticate	(EEwsConnection *cnc,
-						 SoupAuth *auth,
-						 const gchar *user,
-						 const gchar *passwd,
-						 GError *error);
+void		e_ews_connection_queue_request	(EEwsConnection *cnc,
+						 ESoapMessage *msg,
+						 EEwsResponseCallback cb,
+						 gint pri,
+						 GCancellable *cancellable,
+						 GSimpleAsyncResult *simple);
 
-typedef void (*EEwsAutoDiscoverCallback) (EwsUrls *urls, gpointer user_data, GError *error);
-void		e_ews_autodiscover_ws_url	(EEwsAutoDiscoverCallback cb,
-						 gpointer cbdata,
-						 const gchar *email,
+gboolean	e_ews_autodiscover_ws_url_sync	(CamelEwsSettings *settings,
+						 const gchar *email_address,
 						 const gchar *password,
-						 const gchar *ews_url,
-						 const gchar *username);
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_autodiscover_ws_url	(CamelEwsSettings *settings,
+						 const gchar *email_address,
+						 const gchar *password,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_autodiscover_ws_url_finish
+						(CamelEwsSettings *settings,
+						 GAsyncResult *result,
+						 GError **error);
+const gchar *	e_ews_connection_get_mailbox	(EEwsConnection *cnc);
 void		e_ews_connection_set_mailbox	(EEwsConnection *cnc,
 						 const gchar *email);
-/* Sync folder items */
-void		e_ews_connection_sync_folder_items_start
-						(EEwsConnection *cnc,
-						 gint pri,
-						 const gchar *sync_state,
-						 const gchar *fid,
-						 const gchar *props,
-						 const gchar *additional_props,
-						 guint max_entries,
-						 GAsyncReadyCallback cb,
-						 GCancellable *cancellable,
-						 gpointer user_data);
 
-gboolean	e_ews_connection_sync_folder_items_finish
-						(EEwsConnection *cnc,
-						 GAsyncResult *result,
-					 	 gchar **sync_state,
-						 gboolean *includes_last_item,
-						 GSList **items_created,
-						 GSList **items_updated,
-						 GSList **items_deleted,
-						 GError **error);
-gboolean	e_ews_connection_sync_folder_items
+void		ews_user_id_free		(EwsUserId *id);
+void		ews_delegate_info_free		(EwsDelegateInfo *info);
+
+void		e_ews_connection_sync_folder_items
 						(EEwsConnection *cnc,
 						 gint pri,
-						 gchar **sync_state,
+						 const gchar *old_sync_state,
 						 const gchar *fid,
 						 const gchar *default_props,
 						 const gchar *additional_props,
 						 guint max_entries,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_sync_folder_items_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 gchar **new_sync_state,
+						 gboolean *includes_last_item,
+						 GSList **items_created,
+						 GSList **items_updated,
+						 GSList **items_deleted,
+						 GError **error);
+gboolean	e_ews_connection_sync_folder_items_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *old_sync_state,
+						 const gchar *fid,
+						 const gchar *default_props,
+						 const gchar *additional_props,
+						 guint max_entries,
+						 gchar **new_sync_state,
 						 gboolean *includes_last_item,
 						 GSList **items_created,
 						 GSList **items_updated,
@@ -176,28 +274,62 @@ gboolean	e_ews_connection_sync_folder_items
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Get folder items */
-void		e_ews_connection_get_items_start
+typedef void	(*EwsConvertQueryCallback)	(ESoapMessage *msg,
+						 const gchar *query,
+						 EEwsFolderType type);
+
+void		e_ews_connection_find_folder_items
 						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *fid,
+						 const gchar *props,
+						 EwsAdditionalProps *add_props,
+						 EwsSortOrder *sort_order,
+						 const gchar *query,
+						 EEwsFolderType type,
+						 EwsConvertQueryCallback convert_query_cb,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_find_folder_items_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 gboolean *includes_last_item,
+						 GSList **items,
+						 GError **error);
+gboolean	e_ews_connection_find_folder_items_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *fid,
+						 const gchar *default_props,
+						 EwsAdditionalProps *add_props,
+						 EwsSortOrder *sort_order,
+						 const gchar *query,
+						 EEwsFolderType type,
+						 gboolean *includes_last_item,
+						 GSList **items,
+						 EwsConvertQueryCallback convert_query_cb,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_get_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
 						 const gchar *default_props,
 						 const gchar *additional_props,
 						 gboolean include_mime,
 						 const gchar *mime_directory,
-						 GAsyncReadyCallback cb,
 						 ESoapProgressFn progress_fn,
 						 gpointer progress_data,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_get_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **items,
 						 GError **error);
-
-gboolean	e_ews_connection_get_items	(EEwsConnection *cnc,
+gboolean	e_ews_connection_get_items_sync	(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
 						 const gchar *default_props,
@@ -209,34 +341,22 @@ gboolean	e_ews_connection_get_items	(EEwsConnection *cnc,
 						 gpointer progress_data,
 						 GCancellable *cancellable,
 						 GError **error);
-/* Delete folder items */
-void		e_ews_connection_delete_items_start	(EEwsConnection *cnc,
-					 gint pri,
-					 GSList *ids,
-					 EwsDeleteType delete_type,
-					 EwsSendMeetingCancellationsType send_cancels,
-					 EwsAffectedTaskOccurrencesType affected_tasks,
-					 GAsyncReadyCallback cb,
-					 GCancellable *cancellable,
-					 gpointer user_data);
-
-void		e_ews_connection_delete_item_start	(EEwsConnection *cnc,
-					 gint pri,
-					 EwsId *id,
-					 guint index,
-					 EwsDeleteType delete_type,
-					 EwsSendMeetingCancellationsType send_cancels,
-					 EwsAffectedTaskOccurrencesType affected_tasks,
-					 GAsyncReadyCallback cb,
-					 GCancellable *cancellable,
-					 gpointer user_data);
 
+void		e_ews_connection_delete_items	(EEwsConnection *cnc,
+						 gint pri,
+						 GSList *ids,
+						 EwsDeleteType delete_type,
+						 EwsSendMeetingCancellationsType send_cancels,
+						 EwsAffectedTaskOccurrencesType affected_tasks,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
 gboolean	e_ews_connection_delete_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-
-gboolean	e_ews_connection_delete_items	(EEwsConnection *cnc,
+gboolean	e_ews_connection_delete_items_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 GSList *ids,
 						 EwsDeleteType delete_type,
@@ -245,20 +365,28 @@ gboolean	e_ews_connection_delete_items	(EEwsConnection *cnc,
 						 GCancellable *cancellable,
 						 GError **error);
 
-gboolean
-e_ews_connection_delete_item	(EEwsConnection *cnc,
-				 gint pri,
-				 EwsId *id,
-				 guint index,
-				 EwsDeleteType delete_type,
-				 EwsSendMeetingCancellationsType send_cancels,
-				 EwsAffectedTaskOccurrencesType affected_tasks,
-				 GCancellable *cancellable,
-				 GError **error);
-
-/* Update folder items */
-void		e_ews_connection_update_items_start
+void		e_ews_connection_delete_item	(EEwsConnection *cnc,
+						 gint pri,
+						 EwsId *id,
+						 guint index,
+						 EwsDeleteType delete_type,
+						 EwsSendMeetingCancellationsType send_cancels,
+						 EwsAffectedTaskOccurrencesType affected_tasks,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_delete_item_sync
 						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsId *id,
+						 guint index,
+						 EwsDeleteType delete_type,
+						 EwsSendMeetingCancellationsType send_cancels,
+						 EwsAffectedTaskOccurrencesType affected_tasks,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_update_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *conflict_res,
 						 const gchar *msg_disposition,
@@ -266,17 +394,16 @@ void		e_ews_connection_update_items_start
 						 const gchar *folder_id,
 						 EEwsRequestCreationCallback create_cb,
 						 gpointer create_user_data,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_update_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **ids,
 						 GError **error);
-
-gboolean	e_ews_connection_update_items	(EEwsConnection *cnc,
+gboolean	e_ews_connection_update_items_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *conflict_res,
 						 const gchar *msg_disposition,
@@ -287,53 +414,40 @@ gboolean	e_ews_connection_update_items	(EEwsConnection *cnc,
 						 GSList **ids,
 						 GCancellable *cancellable,
 						 GError **error);
-/* Create folder items */
-void		e_ews_connection_create_items_start
-						(EEwsConnection *cnc,
+
+void		e_ews_connection_create_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *msg_disposition,
 						 const gchar *send_invites,
-						 const gchar *folder_id,
+						 const EwsFolderId *fid,
 						 EEwsRequestCreationCallback create_cb,
 						 gpointer create_user_data,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_create_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **ids,
 						 GError **error);
-
-gboolean	e_ews_connection_create_items	(EEwsConnection *cnc,
+gboolean	e_ews_connection_create_items_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *msg_disposition,
 						 const gchar *send_invites,
-						 const gchar *folder_id,
+						 const EwsFolderId *fid,
 						 EEwsRequestCreationCallback create_cb,
 						 gpointer create_user_data,
 						 GSList **ids,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Sync folder hierarchy */
-gboolean	e_ews_connection_sync_folder_hierarchy
-						(EEwsConnection *cnc,
-						 gint pri,
-						 gchar **sync_state,
-						 gboolean *includes_last_folder,
-						 GSList **folders_created,
-						 GSList **folders_updated,
-						 GSList **folders_deleted,
-						 GCancellable *cancellable,
-						 GError **error);
-void		e_ews_connection_sync_folder_hierarchy_start
+void		e_ews_connection_sync_folder_hierarchy
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *sync_state,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_sync_folder_hierarchy_finish
 						(EEwsConnection *cnc,
@@ -344,16 +458,25 @@ gboolean	e_ews_connection_sync_folder_hierarchy_finish
 						 GSList **folders_updated,
 						 GSList **folders_deleted,
 						 GError **error);
-/* Resolve names */
-void		e_ews_connection_resolve_names_start
+gboolean	e_ews_connection_sync_folder_hierarchy_sync
 						(EEwsConnection *cnc,
+						 gint pri,
+						 gchar **sync_state,
+						 gboolean *includes_last_folder,
+						 GSList **folders_created,
+						 GSList **folders_updated,
+						 GSList **folders_deleted,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_resolve_names	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *resolve_name,
 						 EwsContactsSearchScope scope,
 						 GSList *parent_folder_ids,
 						 gboolean fetch_contact_data,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_resolve_names_finish
 						(EEwsConnection *cnc,
@@ -362,7 +485,8 @@ gboolean	e_ews_connection_resolve_names_finish
 						 GSList **contact_items,
 						 gboolean *includes_last_item,
 						 GError **error);
-gboolean	e_ews_connection_resolve_names	(EEwsConnection *cnc,
+gboolean	e_ews_connection_resolve_names_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *resolve_name,
 						 EwsContactsSearchScope scope,
@@ -373,13 +497,12 @@ gboolean	e_ews_connection_resolve_names	(EEwsConnection *cnc,
 						 gboolean *includes_last_item,
 						 GCancellable *cancellable,
 						 GError **error);
-/* Expand distribution lists */
-void		e_ews_connection_expand_dl_start
-						(EEwsConnection *cnc,
+
+void		e_ews_connection_expand_dl	(EEwsConnection *cnc,
 						 gint pri,
 						 const EwsMailbox *mb,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_expand_dl_finish
 						(EEwsConnection *cnc,
@@ -387,7 +510,7 @@ gboolean	e_ews_connection_expand_dl_finish
 						 GSList **mailboxes,
 						 gboolean *includes_last_item,
 						 GError **error);
-gboolean	e_ews_connection_expand_dl	(EEwsConnection *cnc,
+gboolean	e_ews_connection_expand_dl_sync	(EEwsConnection *cnc,
 						 gint pri,
 						 const EwsMailbox *mb,
 						 GSList **mailboxes,
@@ -395,49 +518,54 @@ gboolean	e_ews_connection_expand_dl	(EEwsConnection *cnc,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/*Create Folder*/
-
-void		e_ews_connection_create_folder_start
+gboolean	e_ews_connection_ex_to_smtp_sync
 						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *name,
+						 const gchar *ex_address,
+						 gchar **smtp_address,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_create_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *parent_folder_id,
 						 gboolean is_distinguished_id,
 						 const gchar *folder_name,
-						 GAsyncReadyCallback cb,
+						 EEwsFolderType folder_type,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_create_folder_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 EwsFolderId **folder_id,
 						 GError **error);
-gboolean	e_ews_connection_create_folder	(EEwsConnection *cnc,
+gboolean	e_ews_connection_create_folder_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *parent_folder_id,
 						 gboolean is_distinguished_id,
 						 const gchar *folder_name,
+						 EEwsFolderType folder_type,
 						 EwsFolderId **folder_id,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/*Delete Folder*/
-
-void		e_ews_connection_delete_folder_start
-						(EEwsConnection *cnc,
+void		e_ews_connection_delete_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean is_distinguished_id,
 						 const gchar *delete_type,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_delete_folder_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-
-gboolean	e_ews_connection_delete_folder	(EEwsConnection *cnc,
+gboolean	e_ews_connection_delete_folder_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean is_distinguished_id,
@@ -445,68 +573,82 @@ gboolean	e_ews_connection_delete_folder	(EEwsConnection *cnc,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Update (e.g. rename) folder */
-void		e_ews_connection_update_folder_start
-						(EEwsConnection *cnc,
+void		e_ews_connection_update_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 EEwsRequestCreationCallback create_cb,
 						 gpointer create_user_data,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_update_folder_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-
-gboolean	e_ews_connection_update_folder	(EEwsConnection *cnc,
+gboolean	e_ews_connection_update_folder_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 EEwsRequestCreationCallback create_cb,
 						 gpointer create_user_data,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Move folder (to new parent; for rename we need update_folder) */
-void		e_ews_connection_move_folder_start
-						(EEwsConnection *cnc,
+void		e_ews_connection_move_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *to_folder,
 						 const gchar *folder,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_move_folder_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-
-gboolean	e_ews_connection_move_folder	(EEwsConnection *cnc,
+gboolean	e_ews_connection_move_folder_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *to_folder,
 						 const gchar *folder,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Copy and Move item */
-void		e_ews_connection_move_items_start
+void		e_ews_connection_get_folder	(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *folder_shape,
+						 EwsAdditionalProps *add_props,
+						 GSList *folder_ids,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_folder_finish
 						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GSList **folders,
+						 GError **error);
+gboolean	e_ews_connection_get_folder_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *folder_shape,
+						 EwsAdditionalProps *add_props,
+						 GSList *folder_ids,
+						 GSList **folders,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_move_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean docopy,
 						 GSList *ids,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_move_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **items,
 						 GError **error);
-
-gboolean	e_ews_connection_move_items	(EEwsConnection *cnc,
+gboolean	e_ews_connection_move_items_sync
+						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean docopy,
@@ -515,23 +657,20 @@ gboolean	e_ews_connection_move_items	(EEwsConnection *cnc,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Create attachemnts and associate them with an item */
-void		e_ews_connection_create_attachments_start
+void		e_ews_connection_create_attachments
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const EwsId *parent,
 						 const GSList *files,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 GSList *	e_ews_connection_create_attachments_finish
 						(EEwsConnection *cnc,
 						 gchar **change_key,
 						 GAsyncResult *result,
 						 GError **error);
-
-GSList *	e_ews_connection_create_attachments
+GSList *	e_ews_connection_create_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const EwsId *parent,
@@ -540,52 +679,46 @@ GSList *	e_ews_connection_create_attachments
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Delete attachemnts */
-void		e_ews_connection_delete_attachments_start
+void		e_ews_connection_delete_attachments
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 GSList *	e_ews_connection_delete_attachments_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-
-GSList *	e_ews_connection_delete_attachments
+GSList *	e_ews_connection_delete_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
 						 GCancellable *cancellable,
 						 GError **error);
 
-/* Get attachments items */
-void		e_ews_connection_get_attachments_start
+void		e_ews_connection_get_attachments
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *comp_uid,
 						 const GSList *ids,
 						 const gchar *cache,
 						 gboolean include_mime,
-						 GAsyncReadyCallback cb,
 						 ESoapProgressFn progress_fn,
 						 gpointer progress_data,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-
 GSList *	e_ews_connection_get_attachments_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **items,
 						 GError **error);
-
-GSList *	e_ews_connection_get_attachments
+GSList *	e_ews_connection_get_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *comp_uid,
-						 GSList *ids,
+						 const GSList *ids,
 						 const gchar *cache,
 						 gboolean include_mime,
 						 GSList **items,
@@ -594,51 +727,52 @@ GSList *	e_ews_connection_get_attachments
 						 GCancellable *cancellable,
 						 GError **error);
 
-void		e_ews_connection_get_oal_list_start
+gboolean	e_ews_connection_get_oal_list_sync
 						(EEwsConnection *cnc,
-						 GAsyncReadyCallback cb,
+						 GSList **oals,
 						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_get_oal_list	(EEwsConnection *cnc,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_get_oal_list_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **oals,
 						 GError **error);
-void		e_ews_connection_get_oal_detail_start
+gboolean	e_ews_connection_get_oal_detail_sync
 						(EEwsConnection *cnc,
 						 const gchar *oal_id,
 						 const gchar *oal_element,
-						 GAsyncReadyCallback cb,
+						 GSList **elements,
 						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_get_oal_detail	(EEwsConnection *cnc,
+						 const gchar *oal_id,
+						 const gchar *oal_element,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_get_oal_detail_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **elements,
 						 GError **error);
-gboolean	e_ews_connection_get_oal_detail 
-						(EEwsConnection *cnc,
-						 const gchar *oal_id,
-						 const gchar *oal_element,
-						 GSList **elements,
-						 GCancellable *cancellable,
-						 GError **error);
 
-/* Get free-busy status */
-void		e_ews_connection_get_free_busy_start
-						(EEwsConnection *cnc,
+void		e_ews_connection_get_free_busy	(EEwsConnection *cnc,
 						 gint pri,
 						 EEwsRequestCreationCallback free_busy_cb,
 						 gpointer free_busy_user_data,
-						 GAsyncReadyCallback cb,
 						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
 						 gpointer user_data);
 gboolean	e_ews_connection_get_free_busy_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **free_busy,
 						 GError **error);
-gboolean	e_ews_connection_get_free_busy
+gboolean	e_ews_connection_get_free_busy_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 EEwsRequestCreationCallback free_busy_cb,
@@ -646,24 +780,188 @@ gboolean	e_ews_connection_get_free_busy
 						 GSList **free_busy,
 						 GCancellable *cancellable,
 						 GError **error);
-void		e_ews_connection_download_oal_file_start
+gboolean	e_ews_connection_download_oal_file_sync
 						(EEwsConnection *cnc,
 						 const gchar *cache_filename,
-						 GAsyncReadyCallback cb,
 						 EwsProgressFn progress_fn,
 						 gpointer progress_data,
 						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_download_oal_file
+						(EEwsConnection *cnc,
+						 const gchar *cache_filename,
+						 EwsProgressFn progress_fn,
+						 gpointer progress_data,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback cb,
 						 gpointer user_data);
-
 gboolean	e_ews_connection_download_oal_file_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GError **error);
-gboolean	e_ews_connection_download_oal_file	
+
+void		e_ews_connection_get_delegate	(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 gboolean include_permissions,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_delegate_finish
 						(EEwsConnection *cnc,
-						 const gchar *cache_filename,
-						 EwsProgressFn progress_fn,
-						 gpointer progress_data,
+						 GAsyncResult *result,
+						 EwsDelegateDeliver *deliver_to,
+						 GSList **delegates, /* EwsDelegateInfo * */
+						 GError **error);
+gboolean	e_ews_connection_get_delegate_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 gboolean include_permissions,
+						 EwsDelegateDeliver *deliver_to,
+						 GSList **delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_add_delegate	(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_add_delegate_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_add_delegate_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_remove_delegate
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const GSList *delegate_ids, /* EwsUserId * */
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_remove_delegate_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_remove_delegate_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const GSList *delegate_ids, /* EwsUserId * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_update_delegate
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 EwsDelegateDeliver deliver_to,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_update_delegate_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_update_delegate_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 EwsDelegateDeliver deliver_to,
+						 const GSList *delegates, /* EwsDelegateInfo * */
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_get_folder_permissions
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_folder_permissions_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GSList **permissions,
+						 GError **error);
+gboolean	e_ews_connection_get_folder_permissions_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 GSList **permissions,
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_set_folder_permissions
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 EEwsFolderType folder_type,
+						 const GSList *permissions,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_set_folder_permissions_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_set_folder_permissions_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 EwsFolderId *folder_id,
+						 EEwsFolderType folder_type,
+						 const GSList *permissions,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_get_password_expiration
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+
+gboolean	e_ews_connection_get_password_expiration_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 gchar **exp_date,
+						 GError **error);
+
+gboolean	e_ews_connection_get_password_expiration_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 gchar **exp_date,
+						 GCancellable *cancellable,
+						 GError **error);
+
+void		e_ews_connection_get_folder_info
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const EwsFolderId *folder_id,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_folder_info_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 EEwsFolder **folder,
+						 GError **error);
+gboolean	e_ews_connection_get_folder_info_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *mail_id,
+						 const EwsFolderId *folder_id,
+						 EEwsFolder **folder,
 						 GCancellable *cancellable,
 						 GError **error);
 
